import { Body, ContentType, Controller, Param, Post, QueryParam, Req, UseBefore } from 'routing-controllers';
import { OpenAPI } from 'routing-controllers-openapi';
import { RunnerGroupNeededError } from '../errors/RunnerErrors';
import RawBodyMiddleware from '../middlewares/RawBody';
import { ImportRunner } from '../models/actions/ImportRunner';
import { ImportSchoolRunner } from '../models/actions/ImportSchoolRunner';
import { ResponseRunner } from '../models/responses/ResponseRunner';
import { RunnerController } from './RunnerController';

@Controller()
//@Authorized("IMPORT:read")
export class ImportController {
    private runnerController: RunnerController;

    /**
     * Gets the repository of this controller's model/entity.
     */
    constructor() {
        this.runnerController = new RunnerController();
    }

    @Post('/runners/import')
    @ContentType("application/json")
    @OpenAPI({ description: "Create new runners from json" })
    async postJSON(@Body({ validate: true, type: ImportSchoolRunner }) importRunners: ImportRunner[], @QueryParam("group") groupID: number) {
        if (!groupID) { throw new RunnerGroupNeededError(); }
        let responseRunners: ResponseRunner[] = new Array<ResponseRunner>();
        for await (let runner of importRunners) {
            responseRunners.push(await this.runnerController.post(await runner.toCreateRunner(groupID)));
        }
        return responseRunners;
    }

    @Post('/organisations/:id/import')
    @ContentType("application/json")
    @OpenAPI({ description: "Create new runners from json" })
    async postOrgsJSON(@Body({ validate: true, type: ImportSchoolRunner }) importRunners: ImportRunner[], @Param('id') id: number) {
        return await this.postJSON(importRunners, id)
    }

    @Post('/teams/:id/import')
    @ContentType("application/json")
    @OpenAPI({ description: "Create new runners from json" })
    async postTeamsJSON(@Body({ validate: true, type: ImportSchoolRunner }) importRunners: ImportRunner[], @Param('id') id: number) {
        return await this.postJSON(importRunners, id)
    }

    @Post('/runners/import/csv')
    @ContentType("text/csv")
    @UseBefore(RawBodyMiddleware)
    @OpenAPI({ description: "Create new runners from csv" })
    postCSV(@Req() request: any) {
        console.log(request.rawBody.toString())
        throw new Error("Not implemented yet.");
    }
}